@@ -140,6 +140,25 @@ class CommonLoggerTestsMixIn(object):
                      'info', 'debug', 'log'):
             self.assertRaises(AttributeError, getattr, log, func)
 
+    @mock.patch('platform.system', return_value='Linux')
+    def test_eventlog_missing(self, platform_mock):
+        self.config(use_eventlog=True)
+        self.assertRaises(RuntimeError,
+                          log._setup_logging_from_conf,
+                          self.CONF,
+                          'test',
+                          'test')
+
+    @mock.patch('platform.system', return_value='Windows')
+    @mock.patch('logging.handlers.NTEventLogHandler')
+    @mock.patch('oslo_log.log.getLogger')
+    def test_eventlog(self, loggers_mock, handler_mock, platform_mock):
+        self.config(use_eventlog=True)
+        log._setup_logging_from_conf(self.CONF, 'test', 'test')
+        handler_mock.assert_called_once_with('test')
+        mock_logger = loggers_mock.return_value.logger
+        mock_logger.addHandler.assert_any_call(handler_mock.return_value)
+
 
 class LoggerTestCase(CommonLoggerTestsMixIn, test_base.BaseTestCase):
     def setUp(self):
@@ -591,6 +610,22 @@ class JSONFormatterTestCase(LogTestBase):
         # convert it using repr() to prevent serialization error on logging.
         self.assertEqual(['repr'], data['args'])
 
+    def test_extra_args_filtered(self):
+        test_msg = 'This is a %(test)s line %%(unused)'
+        test_data = {'test': 'log', 'unused': 'removeme'}
+        self.log.debug(test_msg, test_data)
+
+        data = jsonutils.loads(self.stream.getvalue())
+        self.assertNotIn('unused', data['args'])
+
+    def test_entire_dict(self):
+        test_msg = 'This is a %s dict'
+        test_data = {'test': 'log', 'other': 'value'}
+        self.log.debug(test_msg, test_data)
+
+        data = jsonutils.loads(self.stream.getvalue())
+        self.assertEqual(test_data, data['args'])
+
 
 def get_fake_datetime(retval):
     class FakeDateTime(datetime.datetime):
@@ -1702,6 +1737,20 @@ class LogConfigTestCase(BaseTestCase):
                                            disable_existing_loggers=False)
 
 
+class SavingAdapter(log.KeywordArgumentAdapter):
+
+    def __init__(self, *args, **kwds):
+        super(log.KeywordArgumentAdapter, self).__init__(*args, **kwds)
+        self.results = []
+
+    def process(self, msg, kwargs):
+        # Run the real adapter and save the inputs and outputs
+        # before returning them so the test can examine both.
+        results = super(SavingAdapter, self).process(msg, kwargs)
+        self.results.append((msg, kwargs, results))
+        return results
+
+
 class KeywordArgumentAdapterTestCase(BaseTestCase):
 
     def setUp(self):
@@ -1754,52 +1803,44 @@ class KeywordArgumentAdapterTestCase(BaseTestCase):
             kwargs)
 
     def test_pass_args_to_log(self):
-        a = log.KeywordArgumentAdapter(self.mock_log, {})
+        a = SavingAdapter(self.mock_log, {})
+
         message = 'message'
         exc_message = 'exception'
-        key = 'name'
         val = 'value'
         a.log(logging.DEBUG, message, name=val, exc_info=exc_message)
-        if six.PY3:
-            self.mock_log._log.assert_called_once_with(
-                logging.DEBUG,
-                message,
-                (),
-                extra={key: val, 'extra_keys': [key]},
-                exc_info=exc_message
-            )
-        else:
-            self.mock_log.log.assert_called_once_with(
-                logging.DEBUG,
-                message,
-                extra={key: val, 'extra_keys': [key]},
-                exc_info=exc_message
-            )
+
+        expected = {
+            'exc_info': exc_message,
+            'extra': {'name': val, 'extra_keys': ['name']},
+        }
+
+        actual = a.results[0]
+        self.assertEqual(message, actual[0])
+        self.assertEqual(expected, actual[1])
+        results = actual[2]
+        self.assertEqual(message, results[0])
+        self.assertEqual(expected, results[1])
 
     def test_pass_args_via_debug(self):
-        a = log.KeywordArgumentAdapter(self.mock_log, {})
+
+        a = SavingAdapter(self.mock_log, {})
         message = 'message'
         exc_message = 'exception'
-        key = 'name'
         val = 'value'
         a.debug(message, name=val, exc_info=exc_message)
-        # The adapter implementation for debug() is different for
-        # python 3, so we expect a different method to be called
-        # internally.
-        if six.PY3:
-            self.mock_log._log.assert_called_once_with(
-                logging.DEBUG,
-                message,
-                (),
-                extra={key: val, 'extra_keys': [key]},
-                exc_info=exc_message
-            )
-        else:
-            self.mock_log.debug.assert_called_once_with(
-                message,
-                extra={key: val, 'extra_keys': [key]},
-                exc_info=exc_message
-            )
+
+        expected = {
+            'exc_info': exc_message,
+            'extra': {'name': val, 'extra_keys': ['name']},
+        }
+
+        actual = a.results[0]
+        self.assertEqual(message, actual[0])
+        self.assertEqual(expected, actual[1])
+        results = actual[2]
+        self.assertEqual(message, results[0])
+        self.assertEqual(expected, results[1])
 
 
 class UnicodeConversionTestCase(BaseTestCase):
